builtin-push: also ask config for remote information
[git.git] / git-am.sh
index 6ed527c..872145b 100755 (executable)
--- a/git-am.sh
+++ b/git-am.sh
@@ -1,14 +1,13 @@
 #!/bin/sh
 #
-#
+# Copyright (c) 2005, 2006 Junio C Hamano
+
+USAGE='[--signoff] [--dotest=<dir>] [--utf8] [--binary] [--3way]
+  [--interactive] [--whitespace=<option>] <mbox>...
+  or, when resuming [--skip | --resolved]'
 . git-sh-setup
 
-usage () {
-    echo >&2 "usage: $0 [--signoff] [--dotest=<dir>] [--utf8] [--binary] [--3way] <mbox>"
-    echo >&2 " or, when resuming"
-    echo >&2 " $0 [--skip | --resolved]"
-    exit 1;
-}
+git var GIT_COMMITTER_IDENT >/dev/null || exit
 
 stop_here () {
     echo "$1" >"$dotest/next"
@@ -92,13 +91,17 @@ fall_back_3way () {
     # saying that we reverted all those changes.
 
     git-merge-resolve $orig_tree -- HEAD $his_tree || {
+           if test -d "$GIT_DIR/rr-cache"
+           then
+               git-rerere
+           fi
            echo Failed to merge in the changes.
            exit 1
     }
 }
 
 prec=4
-dotest=.dotest sign= utf8= keep= skip= interactive= resolved= binary=
+dotest=.dotest sign= utf8= keep= skip= interactive= resolved= binary= ws=
 
 while case "$#" in 0) break;; esac
 do
@@ -131,6 +134,9 @@ do
        --sk|--ski|--skip)
        skip=t; shift ;;
 
+       --whitespace=*)
+       ws=$1; shift ;;
+
        --)
        shift; break ;;
        -*)
@@ -164,18 +170,16 @@ else
        # Start afresh.
        mkdir -p "$dotest" || exit
 
-       # cat does the right thing for us, including '-' to mean
-       # standard input.
-       cat "$@" |
-       git-mailsplit -d$prec "$dotest/" >"$dotest/last" || {
+       git-mailsplit -d"$prec" -o"$dotest" -b -- "$@" > "$dotest/last" ||  {
                rm -fr "$dotest"
                exit 1
        }
 
# -b, -s, -u and -k flags are kept for the resuming session after
# a patch failure.
        # -3 and -i can and must be given when resuming.
        echo "$binary" >"$dotest/binary"
+       echo " $ws" >"$dotest/whitespace"
        echo "$sign" >"$dotest/sign"
        echo "$utf8" >"$dotest/utf8"
        echo "$keep" >"$dotest/keep"
@@ -203,6 +207,7 @@ if test "$(cat "$dotest/keep")" = t
 then
        keep=-k
 fi
+ws=`cat "$dotest/whitespace"`
 if test "$(cat "$dotest/sign")" = t
 then
        SIGNOFF=`git-var GIT_COMMITTER_IDENT | sed -e '
@@ -218,6 +223,7 @@ this=`cat "$dotest/next"`
 if test "$skip" = t
 then
        this=`expr "$this" + 1`
+       resume=
 fi
 
 if test "$this" -gt "$last"
@@ -232,6 +238,7 @@ do
        msgnum=`printf "%0${prec}d" $this`
        next=`expr "$this" + 1`
        test -f "$dotest/$msgnum" || {
+               resume=
                go_next
                continue
        }
@@ -256,6 +263,13 @@ do
        GIT_AUTHOR_NAME="$(sed -n '/^Author/ s/Author: //p' "$dotest/info")"
        GIT_AUTHOR_EMAIL="$(sed -n '/^Email/ s/Email: //p' "$dotest/info")"
        GIT_AUTHOR_DATE="$(sed -n '/^Date/ s/Date: //p' "$dotest/info")"
+
+       if test -z "$GIT_AUTHOR_EMAIL"
+       then
+               echo "Patch does not have a valid e-mail address."
+               stop_here $this
+       fi
+
        export GIT_AUTHOR_NAME GIT_AUTHOR_EMAIL GIT_AUTHOR_DATE
 
        SUBJECT="$(sed -n '/^Subject/ s/Subject: //p' "$dotest/info")"
@@ -292,7 +306,7 @@ do
            } >"$dotest/final-commit"
            ;;
        *)
-               case "$resolved,$interactive" in
+               case "$resolved$interactive" in
                tt)
                        # This is used only for interactive view option.
                        git-diff-index -p --cached HEAD >"$dotest/patch"
@@ -347,7 +361,7 @@ do
 
        case "$resolved" in
        '')
-               git-apply $binary --index "$dotest/patch"
+               git-apply $binary --index $ws "$dotest/patch"
                apply_status=$?
                ;;
        t)
@@ -356,6 +370,19 @@ do
                # trust what the user has in the index file and the
                # working tree.
                resolved=
+               changed="$(git-diff-index --cached --name-only HEAD)"
+               if test '' = "$changed"
+               then
+                       echo "No changes - did you forget update-index?"
+                       stop_here $this
+               fi
+               unmerged=$(git-ls-files -u)
+               if test -n "$unmerged"
+               then
+                       echo "You still have unmerged paths in your index"
+                       echo "did you forget update-index?"
+                       stop_here $this
+               fi
                apply_status=0
                ;;
        esac
@@ -366,7 +393,7 @@ do
                then
                    # Applying the patch to an earlier tree and merging the
                    # result may have produced the same tree as ours.
-                   changed="$(git-diff-index --cached --name-only -z HEAD)"
+                   changed="$(git-diff-index --cached --name-only HEAD)"
                    if test '' = "$changed"
                    then
                            echo No changes -- Patch already applied.
@@ -164,18 +170,16 @@ else
        }
 
-       # -b, -s, -u and -k flags are kept for the resuming session after
-       # a patch failure.
+       # -b, -s, -u, -k and --whitespace flags are kept for the
+       # resuming session after a patch failure.
        # -3 and -i can and must be given when resuming.
        echo "$binary" >"$dotest/binary"